/*
	*
	* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>
#include <QThread>
#include <QString>

#include <desq/Power.hpp>

class DesQSession : public QThread {
	Q_OBJECT

	public:
		enum SessionType {
			DESQ = 0x715982,				// A standard DesQ session with DesQ Shell
			OYSTER							// Run the Oyster Shell in place of DesQ Shell
		};

		/* Init */
		DesQSession( DesQSession::SessionType mType = DesQSession::DESQ );

		bool isRunning();

	public Q_SLOTS:
		/* Start the session by performing various tasks */
		void start( QString session = QString() );

		/* Stop the session by saving the state */
		void stop();

		/* Intitiate quit */
		void handleMessages( const QString );

		/* Initiate hibernate  */
        inline void hibernate() {

			handleMessages( "hibernate" );
		};

		/* Initiate lock */
        inline void lock() {

			handleMessages( "lock" );
		};

		/* Initiate logout */
        inline void logout() {

			handleMessages( "logout" );
		};

		/* Initiate reboot */
        inline void reboot() {

			handleMessages( "reboot" );
		};

		/* Initiate shutdown */
        inline void shutdown() {

			handleMessages( "shutdown" );
		};

		/* Initiate suspend */
        inline void suspend() {

			handleMessages( "suspend" );
		};

	protected:
		void run();

	private:
		/* Setup the global environmental vriables like SESSION, etc */
		void setupEnvironmentVars();

		/* Start DesQ Shell */
		void startDesQShell();

		/* Start desktop: @desktop - full path to .desktop file */
		void startDesktop( QString desktop );

		/* Start scripts: @script - full path to the script/exec to be run */
		void startScript( QString script );

		/* Enable num lock */
		void enableNumLock();

		/* Enable num lock */
		void restoreSession( QString name = "Previous" );

		/* Interface for @class Power */
		Power *pwr;

		/* Session name and type */
		int mSessionType = 0;
		QString mSessionName;

		/* PID of wayfire and shell */
		qint64 wfPID, shellPID;

		/* QSettings instance */
		QSettings *session;

		/* Check */
		bool registered = true;
};
