/*
	*.* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QDir>

#include <desq/DesQUtils.hpp>

#include "DesQCoreApplication.hpp"
#include "DesQSession.hpp"

int main( int argc, char *argv[] ) {

	qInstallMessageHandler( Logger );

    QCoreApplication app( argc, argv );

    // Set application info
    app.setOrganizationName( "DesQ" );
    app.setApplicationName( "DesQSession" );

	DesQSession *session;
	if ( app.arguments().contains( "--oyster" ) )
		session = new DesQSession( DesQSession::OYSTER );

	else
		session = new DesQSession( DesQSession::DESQ );

	if ( session->isRunning() ) {
		QDBusInterface iface( "org.desq.Session", "/org/DesQ/Session", "org.desq.Session", QDBusConnection::sessionBus() );

		if ( app.arguments().contains( "--lock" ) )
			iface.call( "lock" );

		else if ( app.arguments().contains( "--logout" ) )
			iface.call( "logout" );

		else if ( app.arguments().contains( "--suspend" ) )
			iface.call( "suspend" );

		else if ( app.arguments().contains( "--hibernate" ) )
			iface.call( "hibernate" );

		else if ( app.arguments().contains( "--shutdown" ) )
			iface.call( "shutdown" );

		else if ( app.arguments().contains( "--reboot" ) )
			iface.call( "reboot" );

		else
			qDebug() << "DesQ Session is already active. Use qdbus to talk to it.";

		return 0;
	}

	qDebug() << "Starting session";
    session->start();

    return app.exec();
};
